var fs = require('fs')
var logger = require('../modules/logger')
var pythonShell = require('python-shell')

exports.deploy = function(options, sockets) {

  logger.log('STARTING DEPLOYMENT')
  sockets.updateStatus('started')

  var file = fs.createWriteStream('./scripts/deploy.buildsheet')

  sockets.updateProgress('Writing parameters to buildsheet..', 1)

  file.write('memory:' + options.memorySize + '\n')
  file.write('memory_max:' + options.memorySize + '\n')
  file.write('email@:' + options.email + '\n')
  file.write('name:' + options.hostname + '\n')
  file.write('SYSTEM_pwd:' + options.password + '\n')
  file.write('HANA_instance:' + options.instance + '\n')
  file.write('HANA_sid:' + options.sid + '\n')
  file.write('ip_address:' + options.ipAddress + '\n')

  sockets.updateProgress('Writing parameters to buildsheet..', 3)

  switch(options.memorySize) {
    case 32768:
      dataLunSize = '20'
      logLunSize = '17'
      sharedLunSize = '65'
      break
    case 65536:
      dataLunSize = '40'
      logLunSize = '33'
      sharedLunSize = '100'
      break
    case 131072:
      dataLunSize = '75'
      logLunSize = '70'
      sharedLunSize = '165'
      break
    case 262144:
      dataLunSize = '90'
      logLunSize = '135'
      sharedLunSize = '300'
      break
    default:
      dataLunSize = '20'
      logLunSize = '17'
      sharedLunSize = '65'
      //TODO: calculate lun sizes
      break
  }

  file.write('Data_lun_size:' + dataLunSize + '\n')
  file.write('Log_lun_size:' + logLunSize + '\n')
  file.write('Shared_lun_size:' + sharedLunSize + '\n')

  sockets.updateProgress('Writing parameters to buildsheet..', 5)

  var targetHost = options.targetHost ? options.targetHost : '828422A_21EB28V'
  var image = options.image ? options.image : 'VMImageSLES12forHANA2'
  var storageTemplate = options.storageTemplate ? options.storageTemplate : 'Pool_SAP_F849'
  var virtualProcessor = options.virtual_processor_min ? options.virtual_processor_min : '1'
  var virtualProcessorMin = options.virtual_processor_min ? options.virtual_processor_min : '1'
  var virtualProcessorMax = options.virtual_processor_min ? options.virtual_processor_min : '4'
  var ProcessorEntitlement = options.entitled_capacity ? options.entitled_capacity : '1'
  var ProcessorEntitlementMin = options.entitled_capacity_min ? options.entitled_capacity_min : '1'
  var ProcessorEntitlementMax = options.entitled_capacity_max ? options.entitled_capacity_max : '4'

  const basicParameters = `vlan:ISV LAN
target_host:` + targetHost + `
image:` + image + `
storage_connectivity_group:Any host, all VIOS
storage_template:` + storageTemplate + `
virtual_processor: ` + virtualProcessor + `
virtual_processor_min: ` + virtualProcessorMin + `
virtual_processor_max: ` + virtualProcessorMax + `
entitled_capacity: ` + ProcessorEntitlement + `
entitled_capacity_min: ` + ProcessorEntitlementMin + `
entitled_capacity_max: ` + ProcessorEntitlementMax + `
memory_min:1024
Data_lun_prefix:ROBERT
Data_lun_count:2
Log_lun_prefix:PIERRE
Log_lun_count:2
Shared_lun_prefix:PATRICK
Shared_lun_count:1
Prod_Nonprod:Prod
  `
  file.write(basicParameters)

  var scriptOptions = {
    pythonOptions: ['-u'],
    scriptPath: './scripts',
    args: [
      '-f',
      './scripts/deploy.buildsheet',
      '-p',
      '10.3.25.201',
      '-u',
      'sap_admin',
      '-P',
      'ibm44sap'
    ]
  }

  sockets.updateProgress('Starting python script..', 10)

  var pyshell = new pythonShell('pvcmkvm_v2.py', scriptOptions)

  pyshell.on('message', function (message) {
    switch (message) {
      case '[Info] Values Read....':
        sockets.updateProgress('Reading parameters..', 2)
        break
      case '[Info] Creating LUNs for HANA db..':
        sockets.updateProgress('Creating LUNs..', 6)
        break
      case '[Info]    Creating Data LUNs':
        sockets.updateProgress('Creating Data LUNs..', 10)
        break
      case '[Info]    Creating Logs LUNs':
        sockets.updateProgress('Creating Logs LUNs..', 14)
        break
      case '[Info]    Creating Shared LUNs':
        sockets.updateProgress('Creating Shared LUNs..', 19)
        break
      case '[Info] Additional volumes for HANA successfully created':
        sockets.updateProgress('Generating activation input script', 25)
        break
      case '[Info] Creating VM':
        sockets.updateProgress('Building Virtual Machine', 40)
        break
      case '[Info] Please wait - Attaching additional Storage....':
        sockets.updateProgress('VM deployed. Attaching additional Storage', 50)
        break
      case '[Info] Volumes successfully attached':
        sockets.updateProgress('Volumes successfully attached', 60)
        break
      case '[Info] running Activation Input Script...':
        sockets.updateProgress('Running Activation Input Script', 70)
        break
      case '[Info] VM Rebooting ...':
        sockets.updateProgress('VM Rebooting', 75)
        break
      case '[Info] Email sent to recipient with connection information. HANA Database Installing .......':
        sockets.updateProgress('HANA DATABASE INSTALLING', 80)
        break
      case '[Info] Instance Deployment Completed Successfully !!!!':
        sockets.updateProgress('Instance Deployment Completed Successfully', 100)
        break
      default:
        break
    }
    sockets.pushStdout(message)
  })

  pyshell.end(function (err) {
    if (err) {
      throw err
    } else {
      sockets.updateProgress('Deployment successful', 100)
      sockets.updateStatus('waiting')
      console.log('finished')
    }
  })

  file.end()

}
