#!/usr/bin/python
# Program name  : pvcmkvm
# Purpose       : create a virtual machine with PowerVC rest API
# Author        : HG
# Contact       : hgunasin@au1.ibm.com
# Disclaimer    : This programm is provided "as is". Use it at you own risks
# Last update   : 04/11/2016
# Version       : 1.2
# License       : IBM
# Updates       : V1.2  Updated to create file systems separating volumegroups as per  ISSIC recommendations
# Updates       : Changed volumes attachment method to use nova :  patrick_mensac@fr.ibm.com

import httplib
import json
import base64
import os
import sys, getopt
import time
from subprocess import Popen, PIPE

# Function: perform a call to the api
# For pvcmkvm we only need GET and POST
# I'm sure there is a simpler way to do that

def call_api(type_call, host,  api_url, headers, body, normal_response_code,no_response):
  # Some calls do no return anything
  no_reponse=0
  conn=httplib.HTTPSConnection(host)
  conn.request(type_call, api_url, body, headers)
  response = conn.getresponse()
  # GET http call
  if type_call == "GET":
    # Case ok
    if response.status == normal_response_code:
      raw_response = response.read()
      conn.close()
      json_data = json.loads(raw_response)
      return json_data
    # Case ko
    else:
      raw_response = response.read()
      conn.close()
      json_data = json.loads(raw_response)
      return json_data
      print "[ERROR] Response code ",response.status
  # POST http call
  elif type_call == "POST":
    #print "[Info] JSON Body:",body
    if response.status == normal_response_code:
      if no_response == 0:
        raw_response = response.read()
        conn.close()
        json_data = json.loads(raw_response)
        return json_data
      else:
        print "[Info] [OK] Call successful"
        conn.close()
    else:
      if no_response == 0:
        raw_response = response.read()
        conn.close()
        json_data = json.loads(raw_response)
        return json_data
      else:
        conn.close()
      print "[ERROR] Response code ",response.status
  elif type_call == "DELETE":
    if response.status == normal_response_code:
      if no_response == 0:
        raw_response = response.read()
        conn.close()
        json_data = json.loads(raw_response)
        return json_data
      else:
        print "[Info] [OK] Call successful"
        conn.close()
    else:
      if no_response == 0:
        raw_response = response.read()
        conn.close()
        json_data = json.loads(raw_response)
        return json_data
      else:
        conn.close()
      print "[ERROR] Response code ",response.status

# Function: authenticating powervc
# Return a token

def pvcmd_authenticate(host, headers, user, password):
  conn=httplib.HTTPSConnection(host)
  body_auth_header={"auth":{"scope":{"project":{"domain":{"name":"Default"},"name":"SAPHANA"}},"identity":{"password":{"user":{"domain":{"name":"Default"},"password":"<PASSWORD>","name": "<USER>"}},"methods":["password"]}}}
  body_auth_header["auth"]["identity"]["password"]["user"]["password"]=password
  body_auth_header["auth"]["identity"]["password"]["user"]["name"]=user
  conn.request("POST", "/powervc/openstack/identity/v3/auth/tokens", json.dumps(body_auth_header), headers)
  response=conn.getresponse()
  conn.close()
  auth_token=response.getheader('X-Subject-Token')
  return auth_token

# Function: get tenant id
# Return the tenant id

def pvcmd_get_tenant_id(host, headers, user, password):
  all_tenants=call_api("GET",host,"/powervc/openstack/identity/v3/projects",headers,"",200,0)
  #tenant_id=all_tenants["projects"][0]["id"]
  tenant_id="5d91abd7be654ed99ef0cfd9087ea2ab"
  #return tenant_id

# Function: print help
# Return the 0

def pvcmd_print_help():
  print "[Help] cleanupDemo.py -f <file> -p <powervc_url> -u <user> -P <password>"
  print "[Help] Example: ./cleanupDemo.py -f test.vm. -p localhost -u root -P <PASSWORD>"
  sys.exit(0)

def main(argv):
  # Openstack and PowerVC rest API url

  powervc_image="/powervc/openstack/compute/v2/<TENANT_ID>/images"
  powervc_network="/powervc/openstack/network/v2.0/networks"
  powervc_placement="/powervc/openstack/compute/v2/<TENANT_ID>/os-aggregates"
  powervc_scg="/powervc/openstack/compute/v2/<TENANT_ID>/storage-connectivity-groups"
  powervc_hypervisors="/powervc/openstack/compute/v2.1/<TENANT_ID>/os-hypervisors"
  powervc_storage_template="/powervc/openstack/volume/v2/<TENANT_ID>/types"
  powervc_tenant="/powervc/openstack/identity/v2.0/tenants"
  powervc_server="/powervc/openstack/compute/v2/<TENANT_ID>/servers"
  powervc_server_attvol="/powervc/openstack/compute/v2.1/<TENANT_ID>/servers/<SERVER_ID>/os-volume_attachments"
  powervc_volume_v1="/powervc/openstack/volume/v1/<TENANT_ID>/volumes"
  powervc_volume_v2="/powervc/openstack/volume/v2/<TENANT_ID>/volumes"
  powervc_image="/powervc/openstack/compute/v2/<TENANT_ID>/images"
  powervc_server_list="/powervc/openstack/compute/v2.1/<TENANT_ID>/servers"
  powervc_server_list_detail="/powervc/openstack/compute/v2.1/<TENANT_ID>/servers/detail"


  # Openstack and PowerVC rest API Auth Section

  auth_headers={"Vary": "X-Auth-Token, X-Subject-Token", "Content-Type":"application/json", "Accept":"application/json"}

  # Set input variables from command line

  try:
    opts, args=getopt.getopt(argv,"f:h:p:P:u:")
  except getopt.GetoptError as err:
    print 'pvcmd -f <file>'
    print str(err)
    sys.exit(2)
  for opt, arg in opts:
    if opt in ("-h", "--help"):
      pvcmd_print_help()
    elif opt in ("-f", "--file"):
      file=arg
    elif opt in ("-p"):
      powervc_url=arg
    elif opt in ("-P"):
      password=arg
    elif opt in ("-u"):
      user=arg

  # Read in all settings from "test.vm" stanza file

  for line in open(file):

    stanza=line.split(':')
    if ( stanza[0] == "memory" ):
      f_mem=stanza[1].rstrip()
    elif ( stanza[0] == "memory_max" ):
      f_mem_max=stanza[1].rstrip()
    elif ( stanza[0] == "email@" ):
      f_email=stanza[1].rstrip()
    elif ( stanza[0] == "name" ):
      f_name=stanza[1].rstrip()
    elif ( stanza[0] == "SYSTEM_pwd" ):
      f_systempwd=stanza[1].rstrip()
    elif ( stanza[0] == "HANA_instance" ):
      f_instance=stanza[1].rstrip()
    elif ( stanza[0] == "HANA_sid" ):
      f_sid=stanza[1].rstrip()
    elif ( stanza[0] == "ip_address" ):
      f_ip=stanza[1].rstrip()
    elif ( stanza[0] == "vlan" ):
      f_vlan=stanza[1].rstrip()
    elif ( stanza[0] == "target_host" ):
      f_host=stanza[1].rstrip()
    elif ( stanza[0] == "image" ):
      f_image=stanza[1].rstrip()
    elif ( stanza[0] == "storage_connectivity_group" ):
      f_scg=stanza[1].rstrip()
    elif ( stanza[0] == "virtual_processor_min" ):
      f_vp_min=stanza[1].rstrip()
    elif ( stanza[0] == "virtual_processor_max" ):
      f_vp_max=stanza[1].rstrip()
    elif ( stanza[0] == "virtual_processor" ):
      f_vp=stanza[1].rstrip()
    elif ( stanza[0] == "entitled_capacity_min" ):
      f_ec_min=stanza[1].rstrip()
    elif ( stanza[0] == "entitled_capacity_max" ):
      f_ec_max=stanza[1].rstrip()
    elif ( stanza[0] == "entitled_capacity" ):
      f_ec=stanza[1].rstrip()
    elif ( stanza[0] == "memory_min" ):
      f_mem_min=stanza[1].rstrip()
    elif ( stanza[0] == "storage_template" ):
      f_st=stanza[1].rstrip()
    elif ( stanza[0] == "Data_lun_prefix" ):
      f_Data_lun_prefix=stanza[1].rstrip()
    elif ( stanza[0] == "Data_lun_size" ):
      f_Data_lun_size=stanza[1].rstrip()
    elif ( stanza[0] == "Data_lun_count" ):
      f_Data_lun_count=stanza[1].rstrip()
    elif ( stanza[0] == "Log_lun_prefix" ):
      f_Log_lun_prefix=stanza[1].rstrip()
    elif ( stanza[0] == "Log_lun_size" ):
      f_Log_lun_size=stanza[1].rstrip();
    elif ( stanza[0] == "Log_lun_count" ):
      f_Log_lun_count=stanza[1].rstrip()
    elif ( stanza[0] == "Shared_lun_prefix" ):
      f_Shared_lun_prefix=stanza[1].rstrip()
    elif ( stanza[0] == "Shared_lun_size" ):
      f_Shared_lun_size=stanza[1].rstrip()
    elif ( stanza[0] == "Shared_lun_count" ):
      f_Shared_lun_count=stanza[1].rstrip()
    elif ( stanza[0] == "Prod_Nonprod" ):
      f_prod_np=stanza[1].rstrip()

  # Print all build information provided by build sheet

  print "[Info] Values Read...."
  print "------------------------------------------------------------------"
  print "[Info] IP_address             : ",f_ip.rstrip()
  print "[Info] VLAN                   : ",f_vlan.rstrip()
  print "------------------------------------------------------------------"


  # Getting the token and building the header with the token

  token=pvcmd_authenticate(powervc_url,auth_headers,user,password)
  auth_headers_tok={"X-Auth-Token":token, "Content-Type":"application/json", "Accept":"application/json"}

  #tenant_id=pvcmd_get_tenant_id(powervc_url,auth_headers_tok,user,password)
  tenant_id="5d91abd7be654ed99ef0cfd9087ea2ab"

  # Replacing the tenant id in needed API url

  powervc_image_replaced=powervc_image.replace("<TENANT_ID>",tenant_id)
  powervc_server_replaced=powervc_server.replace("<TENANT_ID>",tenant_id)
  powervc_server_attvol_replaced=powervc_server_attvol.replace("<TENANT_ID>",tenant_id)
  powervc_placement_replaced=powervc_placement.replace("<TENANT_ID>",tenant_id)
  powervc_scg_replaced=powervc_scg.replace("<TENANT_ID>",tenant_id)
  powervc_hypervisors_replaced=powervc_hypervisors.replace("<TENANT_ID>",tenant_id)
  powervc_storage_template_replaced=powervc_storage_template.replace("<TENANT_ID>",tenant_id)
  powervc_volume_v2_replaced=powervc_volume_v2.replace("<TENANT_ID>",tenant_id)
  powervc_server_list_replaced=powervc_server_list.replace("<TENANT_ID>",tenant_id) 
  powervc_server_list_detail_replaced=powervc_server_list_detail.replace("<TENANT_ID>",tenant_id)

  # searching which server id to delete
  # Using nova to search VM id
  #server_ip=f_ip.rstrip()
  #print server_ip
  #varsetup="source /opt/ibm/powervc/powervcrc.pat"
  #executeattach="source /opt/ibm/powervc/powervcrc.pat; /usr/bin/nova --os-username=sap_admin list --ip " +server_ip+" | cut -d '|' -f2 | grep -v +| grep -v ID"
  #import subprocess
  #subprocess.call(["ssh", "root@10.3.25.201", executeattach]);

  # searching which server id to delete

  all_servers_detail=call_api("GET",powervc_url, powervc_server_list_detail_replaced,auth_headers_tok,"",200,0)
  print all_servers_detail
  for a_server in all_servers_detail["servers"]:
     server_vlan=a_server["addresses"]
     for servers in server_vlan["ISV LAN"]:
        if servers["addr"] == f_ip.rstrip():
         print a_server
         server_id=a_server["id"]
         print "[Et voila le server id to delete]", server_id

  #all_servers=call_api("GET",powervc_url, powervc_server_list_replaced,auth_headers_tok,"",200,0)
  #all_servers_detail=call_api("GET",powervc_url, powervc_server_list_detail_replaced,auth_headers_tok,"",200,0)
  #print all_servers_detail
  #for a_server in all_servers_detail["servers"]:
  #   print a_server
  #   for server_vlan in a_server["addresses"]:
  #    server_id=a_server["id"]
  #    print server_id
  #    print server_vlan
     #for servers in server_vlan["addresses"]:
     #for servers in a_server["addresses"]:
     #if server_vlan['ISV VLAN'] == f_ip.rstrip():
     # print a_server
     # server_id=a_server['id']
     #for server_addr in a_server["addresses"]:
     #  print server_addr[addr]
     #if a_server['addresses'] == f_ip.rstrip():
     #server_id=a_server["id"]
     #print "[Et voila le server id to delete]", server_id



  # Searching for volumes attached to the server to delete 
  powervc_volume_attached="/powervc/openstack/compute/v2/<TENANT_ID>/servers/<SERVER_ID>/os-volume_attachments"
  powervc_volume_attached_replaced=powervc_volume_attached.replace("<TENANT_ID>",tenant_id)
  powervc_volume_attached_replaced1=powervc_volume_attached_replaced.replace("<SERVER_ID>",server_id)
  volume_list=""
  all_volumes=call_api("GET",powervc_url, powervc_volume_attached_replaced1, auth_headers_tok,"",200,0)
  print all_volumes
  # searching data volumes..... and do not delete boot volume
  for vol in all_volumes["volumeAttachments"]:
    vol_id=vol["id"]
    if vol["bootIndex"] != 0:
     print "[Info] here are the volumes ids to delete", vol_id
     volume_list=volume_list+" "+vol_id
    #delete_volumes=call_api("DELETE",powervc_url, powervc_volume_v2_replaced + "/" + str(vol_id), auth_headers_tok,"",200,0)

  # Delete  VM based on input values 
  # delete_vm=call_api("DELETE",powervc_url,powervc_server_replaced + "/" + str(server_id),auth_headers_tok,"",200,0)
  # Using nova to delete VM 
    varsetup="source /opt/ibm/powervc/powervcrc.pat"
    executeattach="source /opt/ibm/powervc/powervcrc.pat; /usr/bin/nova --os-username=sap_admin delete "+server_id+" "
    import subprocess
    subprocess.call(["ssh", "root@10.3.25.201", executeattach]);

  time.sleep(60)
  # Using cinder to delete volumes
  f=open("/tmp/_vols.txt","w")
  f.write(volume_list)
  f.close()
  vols = volume_list.split()
  for lun_id in vols:
    print "[Info] volume id = ",lun_id
    varsetup="source /opt/ibm/powervc/powervcrc.pat"
    executeattach="source /opt/ibm/powervc/powervcrc.pat; /usr/bin/cinder --os-username=sap_admin delete "+lun_id+" "
    import subprocess
    subprocess.call(["ssh", "root@10.3.25.201", executeattach]);




  print "[Info] VM deleted sucessfully!!!!"
if __name__ == "__main__":
  main(sys.argv[1:])
