#!/bin/sh
# Purpose     : Customize SLES12 SP1 for SAP HANA platform V2 
# Program name: Activation input file hanav2.sh 
# Author      : PM
# Contact     : patrick_mensac@fr.ibm.com
# Disclaimer  : This programm is provided "as is". Use it at you own risks
# Last update :  07/01/2017
# Version     : 1.0
# License     : IBM
# Updates     :

 # usermod -U root
 ################ Parameters ###############
 
 mem_size=
 nb_of_log_luns=2
 log_disk_size=
 nb_of_data_luns=2
 data_disk_size=
 nb_of_shared_luns=1
 shared_disk_size=
 lv_stripe_size=256
 HANA_SID=
 HANA_instance=
 sidadmpassword=ibm44sap
 SYSTEMpassword=
 maildelivery=
 
 ###########################################
 # Enable at.service
 systemctl enable atd.service
 systemctl start atd.service

function check_os_distrib {
ostype=$(grep ^ID= /etc/os-release|cut -d"=" -f2)
[[ $ostype = \"sles\" || $ostype = \"sles_sap\" ]] && distrib="sles"
[[ $ostype = \"rhel\" ]] && distrib="rhel"
}

function scan_pvs {
ostype=$(grep ^ID= /etc/os-release|cut -d"=" -f2)
[[ $ostype = \"sles\" || $ostype = \"sles_sap\" ]] && rescan-scsi-bus.sh > /dev/null
[[ $ostype = \"rhel\" ]] && pvscan > /dev/null
}


function multipath_reconfigure {
[[ -f /etc/multipath.conf ]] && mv /etc/multipath.conf /etc/multipath.conf.ori
cat >> /etc/multipath.conf << EOF
defaults {
        path_selector           "service-time 0"
        path_grouping_policy    group_by_prio
        prio                    alua
        path_checker            tur
        rr_weight               priorities
        failback                immediate
        no_path_retry           fail
        user_friendly_names     yes
        rr_min_io_rq            10
        dev_loss_tmo            120
}
EOF
systemctl stop multipathd
/sbin/multipath -F
/sbin/multipath -v2 -l
systemctl start multipathd
dracut --force --add multipath
multipath -r
}

# select disks for LOGS 
function disks_for_logs { 
   log_disk_tempo=`multipath -l | grep -v status | grep -v running |awk '{print $1 " " $2}' | grep -w $log_disk_size  -B 1| grep -v - | grep -v $log_disk_size | grep -v "$rootdisk"`
   log_disk_mapper_list=`echo $log_disk_tempo`
   echo "" > /tmp/log_disk_list
   for fi in $(echo $log_disk_mapper_list)
      do 
        echo "/dev/mapper/$fi" | grep -v '(' >> /tmp/log_disk_list
      done
	  for disks in $(cat /tmp/log_disk_list)
	     do 
		  pvcreate $disks 
		 done
   }
   
# select disks for DATA
function disks_for_data {
    data_disk_tempo=`multipath -l | grep -v status | grep -v running | awk '{print $1 " " $2}' | grep -w $data_disk_size  -B 1 | grep -v - | grep -v $data_disk_size | grep -v $rootdisk`
    data_disk_mapper_list=`echo $data_disk_tempo`
    echo "" > /tmp/data_disk_list
    for fi in $(echo $data_disk_mapper_list)
       do
         echo "/dev/mapper/$fi" | grep -v '(' >> /tmp/data_disk_list 
	   done
	   for disks in $(cat /tmp/data_disk_list)
	     do 
		  pvcreate $disks
		 done
	   }
	   
# select disks for SHARED
function disks_for_shared {
    shared_disk_tempo=`multipath -l | grep -v status | grep -v running | awk '{print $1 " " $2}' | grep -w $shared_disk_size  -B 1 | grep -v - | grep -v $shared_disk_size | grep -v $rootdisk`
    shared_disk_mapper_list=`echo $shared_disk_tempo`
    echo "" > /tmp/shared_disk_list 
    for fi in $(echo $shared_disk_mapper_list)
       do
         echo "/dev/mapper/$fi" | grep -v '(' >> /tmp/shared_disk_list 
	   done
	   for disks in $(cat /tmp/shared_disk_list)
	     do 
		  pvcreate $disks
		 done
	   }
	
# Create VGs	
function creating_vgs {
   vgcreate -s 4M --dataalignment 1M hanadatavg $(cat /tmp/data_disk_list)
   vgcreate -s 4M --dataalignment 1M hanalogvg $(cat /tmp/log_disk_list)
   vgcreate -s 4M --dataalignment 1M hanasharedvg $(cat /tmp/shared_disk_list)
   }

# Calculate FS Sizes based on memory size
function calculate_fs_size {
	fs_usrsap_size=30
	fs_shared_size=$(echo $mem_size | awk '{print $1 / 1024 * 1.2}' | awk '{printf("%d\n",$0+=$0<0?-0.5:0.5)}')
	# fs_log_size=$(( round((mem_size / 1024) * 0.5 ) ))
	fs_log_size=$(echo $mem_size | awk '{print $1 / 1024 * 0.5}' | awk '{printf("%d\n",$0+=$0<0?-0.5:0.5)}')
	# fs_data_size=$(( round((mem_size / 1024) * 1.0 ) ))
	fs_data_size=$(echo $mem_size | awk '{print $1 / 1024 * 0.5}' | awk '{printf("%d\n",$0+=$0<0?-0.5:0.5)}')
	}
	
# Create LVs
   function create_lvs {
    no_of_data_luns=`cat /tmp/data_disk_list|wc -w`
    no_of_log_luns=`cat /tmp/log_disk_list|wc -w`
    no_of_shared_luns=`cat /tmp/shared_disk_list|wc -w`
    _data_stripe_tmp=$(echo $data_disk_mapper_list | wc -w )
    _data_stripe=$((_data_stripe_tmp / 2))
    _log_stripe_tmp=$(echo $log_disk_mapper_list | wc -w )
    _log_stripe=$((_log_stripe_tmp / 2))


    lvcreate -i $no_of_data_luns -I ${lv_stripe_size}K -L ${fs_data_size}G hanadatavg -n hanadatalv
    lvcreate -i $no_of_log_luns -I ${lv_stripe_size}K -L ${fs_log_size}G hanalogvg -n hanaloglv
    lvcreate -L ${fs_usrsap_size}G  hanasharedvg -n usrsaplv
    lvcreate -l 100%FREE  hanasharedvg -n hanasharedlv 
   }
# Create FS (xfs)
function create_fs {
   mkfs.xfs -f /dev/hanadatavg/hanadatalv
   mkfs.xfs -f /dev/hanalogvg/hanaloglv
   mkfs.xfs -f /dev/hanasharedvg/usrsaplv
   mkfs.xfs -f /dev/hanasharedvg/hanasharedlv
   }
   
# create mount points 
function create_mount_points {   
   mkdir /usr/sap
   mkdir -p /hana/log
   mkdir -p /hana/data
   mkdir -p /hana/shared
    # Set Permissions
   chmod -R 755 /hana
   }
   
# Add /etc/fstab Entries
function add_fstab_entries {
   echo "# SAP HANA Filesystem fstab Entries"  >> /etc/fstab
   echo "/dev/hanasharedvg/usrsaplv         /usr/sap         xfs        defaults              0 0" >> /etc/fstab
   echo "/dev/hanalogvg/hanaloglv           /hana/log        xfs        defaults              0 0" >> /etc/fstab
   echo "/dev/hanadatavg/hanadatalv         /hana/data       xfs        defaults              0 0" >> /etc/fstab
   echo "/dev/hanasharedvg/hanasharedlv     /hana/shared     xfs        defaults              0 0" >> /etc/fstab
   }
# Mount all file systems
function mount_all {
    /bin/mount -a
	}
	
# send email for completion 
function emailing { 
    useradd -m PowerVC
    DATE=$(date)
    HOST=$(hostname)
    # maildelivery=$(echo $maildelivery | cut -d "/" -f2 )
    msg="$DATE  The system is up. VM can be accessed using IP@:$(ifconfig eth0 | grep inet | grep -v inet6 | awk '{print $2}' | cut -c 6-) user=root password=${sidadmpassword} ----  SAP HANA platform SID=${HANA_SID} and HANA instance number=${HANA_instance}"
    su - PowerVC  -c "echo $msg | mailx -vv -s 'Deployment Through PowerVC of VM ${HOST} is successfull' $(echo $maildelivery | cut -d "/" -f2 )"
   }
   
# update hostfile 
function update_hostfile {
   ipaddr=$(ip a s eth0 | grep "global eth0" | awk '{print $2}' | cut -d '/' -f1) 
   echo $ipaddr `hostname`  >> /etc/hosts
   }
     
# install HANA installation script
function hana_install_script {
  ##############
  # nfs server for HANA code installation
  nfsserver=10.3.25.102
  #####################
  echo "" > /etc/init.d/run-once
  cat >> /etc/init.d/run-once << EOF
   #!/bin/sh
   # SAP HANA Installation #
   ipaddr=$(ip a s eth0 | grep "global eth0" | awk '{print $2}' | cut -d '/' -f1)
   echo $(ip a s eth0 | grep "global eth0" | awk '{print $2}' | cut -d '/' -f1) $(hostname)  >> /etc/hosts
   export HANA_SID=
   HANA_instance=
   SYSTEMpassword=
   hanapassword=$SYSTEMpassword
   mount -o nolock -o nfsvers=3 ${nfsserver}:/sapsources /mnt -t nfs
   #cd /mnt/HANA/HANAv2/51051636/DATA_UNITS/HDB_SERVER_LINUX_PPC64LE
   cd /mnt/HANA/HANAv2/51052326-SAP-HANA-Platform-Edition2.0-SPS02-rev20-Linux-on-Power/DATA_UNITS/HDB_SERVER_LINUX_PPC64LE
   ./hdblcm --dump_configfile_template=/tmp/configfile --action=install
   cp /tmp/configfile /tmp/configfile.pat
   hana_sid=\$(echo \$HANA_SID | tr '[:upper:]' '[:lower:]')
   sed  -i -e "s|^sid=$|sid=\$(echo \$HANA_SID | tr '[:upper:]' '[:lower:]')|" /tmp/configfile
   sed  -i -e "s|^root_password=$|root_password=\$hanapassword|" /tmp/configfile
   sed  -i -e "s|^master_password=$|master_password=\$hanapassword|" /tmp/configfile
   sed  -i -e "s|^sapadm_password=$|sapadm_password=\$hanapassword|" /tmp/configfile
   sed  -i -e "s|^password=$|password=\$hanapassword|" /tmp/configfile
   sed  -i -e "s|^system_user_password=$|system_user_password=\${hanapassword}|" /tmp/configfile
   #sed  -i -e "s|^userid=$|userid=1005|" /tmp/configfile
   #sed  -i -e "s|^groupid=$|groupid=1005|" /tmp/configfile
   sed  -i -e "s|^components=$|components=server,client|" /tmp/configfile
   sed  -i -e "s|^component_medium=$|component_medium=/mnt/HANA/HANAv2/51052326-SAP-HANA-Platform-Edition2.0-SPS02-rev20-Linux-on-Power/|" /tmp/configfile
   sed -i -e "s|^component_root=$|component_root=/mnt/HANA/HANAv2/51052326-SAP-HANA-Platform-Edition2.0-SPS02-rev20-Linux-on-Power/|" /tmp/configfile
   ./hdblcm --sid=\${HANA_SID} --number=\${HANA_instance} --configfile=/tmp/configfile --batch
EOF
chmod 777 /etc/init.d/run-once
#insserv /etc/init.d/run-once
}

# run the HANA installation    
function hana_install_run {
   cd /etc/init.d 
   at -f /etc/init.d/run-once now + 1 minutes
   #at -f /etc/init.d/run-once now
   }
   
# main ()

  distrib=

  check_os_distrib

  if [[ $distrib = "rhel" ]]
  then
  multipath -l > /tmp/multipath.output
  rootdisk_temp=$(vgs --noheadings --options pv_name rhel)
  rootdisk=$(echo $rootdisk_temp | awk -F "/" '{ print $4}' | awk -F "-" '{print $1}')
 pvs --noheadings > /tmp/pvsoutput
 vgs --noheadings --options pv_name rhel > /tmp/vgsoutput
 echo $rootdisk > /tmp/rootdisk.txt
  fi

  if [[ $distrib = "sles" || $distrib = "sles_sap" ]]
  then
  multipath -l > /tmp/multipath.output
  rootdisk_temp=$(vgs --noheadings --options pv_name system)
  rootdisk=$(echo $rootdisk_temp | awk -F "/" '{ print $4}' | awk -F "-" '{print $1}')
 pvs --noheadings > /tmp/pvsoutput
 vgs --noheadings --options pv_name system > /tmp/vgsoutput
 echo $rootdisk > /tmp/rootdisk.txt
  fi


   scan_pvs
   multipath_reconfigure
   disks_for_logs
   disks_for_data
   disks_for_shared
   creating_vgs
   calculate_fs_size
   create_lvs
   create_fs
   create_mount_points
   update_hostfile
   add_fstab_entries
   mount_all
   hana_install_script
   hana_install_run
   emailing 
   # reboot   
   
   
   
   
   
   
