#!/usr/bin/python
# Program name  : pvcmkvm
# Purpose       : create a virtual machine with PowerVC rest API
# Author        : HG
# Contact       : hgunasin@au1.ibm.com
# Disclaimer    : This programm is provided "as is". Use it at you own risks
# Last update   : 04/11/2016
# Version       : 1.2
# License       : IBM
# Updates       : V1.2  Updated to create file systems separating volumegroups as per  ISSIC recommendations
# Updates       : Changed volumes attachment method to use nova :  patrick_mensac@fr.ibm.com

import httplib
import json
import base64
import os
import sys, getopt
import time
from subprocess import Popen, PIPE

# Function: perform a call to the api
# For pvcmkvm we only need GET and POST
# I'm sure there is a simpler way to do that

def call_api(type_call, host,  api_url, headers, body, normal_response_code,no_response):
  # Some calls do no return anything
  no_reponse=0
  conn=httplib.HTTPSConnection(host)
  conn.request(type_call, api_url, body, headers)
  response = conn.getresponse()
  # GET http call
  if type_call == "GET":
    # Case ok
    if response.status == normal_response_code:
      raw_response = response.read()
      conn.close()
      json_data = json.loads(raw_response)
      return json_data
    # Case ko
    else:
      raw_response = response.read()
      conn.close()
      json_data = json.loads(raw_response)
      return json_data
      print "[ERROR] Response code ",response.status
  # POST http call
  elif type_call == "POST":
    #print "[Info] JSON Body:",body
    if response.status == normal_response_code:
      if no_response == 0:
        raw_response = response.read()
        conn.close()
        json_data = json.loads(raw_response)
        return json_data
      else:
        print "[Info] [OK] Call successful"
        conn.close()
    else:
      if no_response == 0:
        raw_response = response.read()
        conn.close()
        json_data = json.loads(raw_response)
        return json_data
      else:
        conn.close()
      print "[ERROR] Response code ",response.status

# Function: authenticating powervc
# Return a token

def pvcmd_authenticate(host, headers, user, password):
  conn=httplib.HTTPSConnection(host)
  body_auth_header={"auth":{"scope":{"project":{"domain":{"name":"Default"},"name":"SAPHANA"}},"identity":{"password":{"user":{"domain":{"name":"Default"},"password":"<PASSWORD>","name": "<USER>"}},"methods":["password"]}}}
  body_auth_header["auth"]["identity"]["password"]["user"]["password"]=password
  body_auth_header["auth"]["identity"]["password"]["user"]["name"]=user
  conn.request("POST", "/powervc/openstack/identity/v3/auth/tokens", json.dumps(body_auth_header), headers)
  response=conn.getresponse()
  conn.close()
  auth_token=response.getheader('X-Subject-Token')
  return auth_token

# Function: get tenant id
# Return the tenant id

def pvcmd_get_tenant_id(host, headers, user, password):
  all_tenants=call_api("GET",host,"/powervc/openstack/identity/v3/projects",headers,"",200,0)
  #tenant_id=all_tenants["projects"][0]["id"]
  tenant_id="5d91abd7be654ed99ef0cfd9087ea2ab"
  #return tenant_id

# Function: sed using python
# replace strings in file
# PM july 2017

def inplace_change(filename, old_string, new_string):
    # Safely read the input filename using 'with'
    with open(filename) as f:
        s = f.read()
        if old_string not in s:
            print '"{old_string}" not found in {filename}.'.format(**locals())
            return

    # Safely write the changed content, if found in the file
    with open(filename, 'w') as f:
        print 'Changing "{old_string}" to "{new_string}" in {filename}'.format(**locals())
        s = s.replace(old_string, new_string)
        f.write(s)


# Function: print help
# Return the 0

def pvcmd_print_help():
  print "[Help] pvcmkvm -f <file> -p <powervc_url> -u <user> -P <password>"
  print "[Help] Example: ./pvcmkvm_v2 -f test.vm. -p localhost -u root -P <PASSWORD>"
  sys.exit(0)

def main(argv):
  # Openstack and PowerVC rest API url

  powervc_image="/powervc/openstack/compute/v2/<TENANT_ID>/images"
  powervc_network="/powervc/openstack/network/v2.0/networks"
  powervc_placement="/powervc/openstack/compute/v2/<TENANT_ID>/os-aggregates"
  powervc_scg="/powervc/openstack/compute/v2/<TENANT_ID>/storage-connectivity-groups"
  powervc_hypervisors="/powervc/openstack/compute/v2.1/<TENANT_ID>/os-hypervisors"
  powervc_storage_template="/powervc/openstack/volume/v2/<TENANT_ID>/types"
  powervc_tenant="/powervc/openstack/identity/v2.0/tenants"
  powervc_server="/powervc/openstack/compute/v2/<TENANT_ID>/servers"
  powervc_server_attvol="/powervc/openstack/compute/v2.1/<TENANT_ID>/servers/<SERVER_ID>/os-volume_attachments"
  powervc_volume_v1="/powervc/openstack/volume/v1/<TENANT_ID>/volumes"
  powervc_volume_v2="/powervc/openstack/volume/v2/<TENANT_ID>/volumes"
  powervc_image="/powervc/openstack/compute/v2/<TENANT_ID>/images"


  # Openstack and PowerVC rest API Auth Section

  auth_headers={"Vary": "X-Auth-Token, X-Subject-Token", "Content-Type":"application/json", "Accept":"application/json"}

  # Set input variables from command line

  try:
    opts, args=getopt.getopt(argv,"f:h:p:P:u:")
  except getopt.GetoptError as err:
    print 'pvcmd -f <file>'
    print str(err)
    sys.exit(2)
  for opt, arg in opts:
    if opt in ("-h", "--help"):
      pvcmd_print_help()
    elif opt in ("-f", "--file"):
      file=arg
    elif opt in ("-p"):
      powervc_url=arg
    elif opt in ("-P"):
      password=arg
    elif opt in ("-u"):
      user=arg

  # Read in all settings from "test.vm" stanza file

  for line in open(file):

    stanza=line.split(':')
    if ( stanza[0] == "memory" ):
      f_mem=stanza[1].rstrip()
    elif ( stanza[0] == "memory_max" ):
      f_mem_max=stanza[1].rstrip()
    elif ( stanza[0] == "email@" ):
      f_email=stanza[1].rstrip()
    elif ( stanza[0] == "name" ):
      f_name=stanza[1].rstrip()
    elif ( stanza[0] == "SYSTEM_pwd" ):
      f_systempwd=stanza[1].rstrip()
    elif ( stanza[0] == "HANA_instance" ):
      f_instance=stanza[1].rstrip()
    elif ( stanza[0] == "HANA_sid" ):
      f_sid=stanza[1].rstrip()
    elif ( stanza[0] == "ip_address" ):
      f_ip=stanza[1].rstrip()
    elif ( stanza[0] == "vlan" ):
      f_vlan=stanza[1].rstrip()
    elif ( stanza[0] == "target_host" ):
      f_host=stanza[1].rstrip()
    elif ( stanza[0] == "image" ):
      f_image=stanza[1].rstrip()
    elif ( stanza[0] == "storage_connectivity_group" ):
      f_scg=stanza[1].rstrip()
    elif ( stanza[0] == "virtual_processor_min" ):
      f_vp_min=stanza[1].rstrip()
    elif ( stanza[0] == "virtual_processor_max" ):
      f_vp_max=stanza[1].rstrip()
    elif ( stanza[0] == "virtual_processor" ):
      f_vp=stanza[1].rstrip()
    elif ( stanza[0] == "entitled_capacity_min" ):
      f_ec_min=stanza[1].rstrip()
    elif ( stanza[0] == "entitled_capacity_max" ):
      f_ec_max=stanza[1].rstrip()
    elif ( stanza[0] == "entitled_capacity" ):
      f_ec=stanza[1].rstrip()
    elif ( stanza[0] == "memory_min" ):
      f_mem_min=stanza[1].rstrip()
    elif ( stanza[0] == "storage_template" ):
      f_st=stanza[1].rstrip()
    elif ( stanza[0] == "Data_lun_prefix" ):
      f_Data_lun_prefix=stanza[1].rstrip()
    elif ( stanza[0] == "Data_lun_size" ):
      f_Data_lun_size=stanza[1].rstrip()
    elif ( stanza[0] == "Data_lun_count" ):
      f_Data_lun_count=stanza[1].rstrip()
    elif ( stanza[0] == "Log_lun_prefix" ):
      f_Log_lun_prefix=stanza[1].rstrip()
    elif ( stanza[0] == "Log_lun_size" ):
      f_Log_lun_size=stanza[1].rstrip();
    elif ( stanza[0] == "Log_lun_count" ):
      f_Log_lun_count=stanza[1].rstrip()
    elif ( stanza[0] == "Shared_lun_prefix" ):
      f_Shared_lun_prefix=stanza[1].rstrip()
    elif ( stanza[0] == "Shared_lun_size" ):
      f_Shared_lun_size=stanza[1].rstrip()
    elif ( stanza[0] == "Shared_lun_count" ):
      f_Shared_lun_count=stanza[1].rstrip()
    elif ( stanza[0] == "Prod_Nonprod" ):
      f_prod_np=stanza[1].rstrip()

  # Print all build information provided by build sheet
  #

  print "[Info] Values Read...."
  print "------------------------------------------------------------------"
  print "[Info] email                  : ",f_email.rstrip()
  print "[Info] SYSTEM pwd             : ",f_systempwd.rstrip()
  print "[Info] HANA Instance          : ",f_instance.rstrip()
  print "[Info] HANA SID               : ",f_sid.rstrip()
  print "[Info] Host Name              : ",f_name.rstrip()
  print "[Info] IP_address             : ",f_ip.rstrip()
  print "[Info] Data_lun_size (G)      : ",f_Data_lun_size.rstrip()
  print "[Info] Log_lun_size (G)       : ",f_Log_lun_size.rstrip()
  print "[Info] Shared_lun_size (G)    : ",f_Shared_lun_size.rstrip()
  print "[Info] VLAN                   : ",f_vlan.rstrip()
  print "[Info] Target_host            : ",f_host.rstrip()
  print "[Info] Image                  : ",f_image.rstrip()
  print "[Info] SCG                    : ",f_scg.rstrip()
  print "[Info] Virtual_processor - min: ",f_vp_min.rstrip()
  print "[Info] Virtual_processor - max: ",f_vp_max.rstrip()
  print "[Info] Virtual_processor      : ",f_vp.rstrip()
  print "[Info] Entitled_capacity - min: ",f_ec_min.rstrip()
  print "[Info] Entitled_capacity - max: ",f_ec_max.rstrip()
  print "[Info] Entitled_capacity      : ",f_ec.rstrip()
  print "[Info] Memory - min           : ",f_mem_min.rstrip()
  print "[Info] Memory - max           : ",f_mem_max.rstrip()
  print "[Info] Memory                 : ",f_mem.rstrip()
  print "[Info] Storage_template       : ",f_st.rstrip()
  print "[Info] Data_lun_prefix        : ",f_Data_lun_prefix.rstrip()
  print "[Info] Data_lun_count         : ",f_Data_lun_count.rstrip()
  print "[Info] Log_lun_prefix         : ",f_Log_lun_prefix.rstrip()
  print "[Info] Log_lun_count          : ",f_Log_lun_count.rstrip()
  print "[Info] Shared_prefix          : ",f_Shared_lun_prefix.rstrip()
  print "[Info] Shared_lun_count       : ",f_Shared_lun_count.rstrip()
  print "[Info] Prod_Nonprod           : ",f_prod_np.rstrip()
  print "------------------------------------------------------------------"

  # Check validity of values given
  #
  datasize=float(f_Data_lun_size) * float(f_Data_lun_count)
  logsize=float(f_Log_lun_size) * float(f_Log_lun_count)
  sharedsize=float(f_Shared_lun_size) * float(f_Shared_lun_count)
  memsize=float(f_mem) / 1024


  if ( len(f_name.rstrip()) == 0 or len(f_ip.rstrip()) == 0 or len(f_vlan.rstrip()) == 0 or len(f_host.rstrip()) == 0 or len(f_image.rstrip()) == 0 or len(f_scg.rstrip()) == 0 or len(f_vp_min.rstrip()) == 0 or len(f_vp_max.rstrip()) == 0 or len(f_vp.rstrip()) == 0 or len(f_ec_min.rstrip()) == 0 or len(f_ec_max.rstrip()) == 0 or len(f_ec.rstrip()) == 0 or len(f_mem_min.rstrip()) == 0 or len(f_mem_max.rstrip()) == 0 or len(f_mem.rstrip()) == 0 or len(f_st.rstrip()) == 0 or len(f_Data_lun_prefix.rstrip()) == 0 or len(f_Data_lun_size.rstrip()) == 0 or len(f_Data_lun_count.rstrip()) == 0 or len(f_Log_lun_prefix.rstrip()) == 0 or len(f_Log_lun_size.rstrip()) == 0 or len(f_Log_lun_count.rstrip()) == 0):
    print "[ERROR] Input Values are invalid ... Please validate for NULL/zero Values and Retry"
    sys.exit(1)
  elif ( len(f_name.rstrip()) > 20 or len(f_ip.rstrip()) > 15 or int(f_vp_min.rstrip()) > int(f_vp.rstrip()) or int(f_vp_max.rstrip()) < int(f_vp.rstrip()) or float(f_ec_min.rstrip()) > float(f_ec.rstrip()) or float(f_ec_max.rstrip()) < float(f_ec.rstrip()) or int(f_mem_min.rstrip()) > int(f_mem.rstrip()) or int(f_mem_max.rstrip()) < int(f_mem.rstrip()) ):
    print "[ERROR] Invalid Range Given for Input Values......."
    sys.exit(1)
  elif ( f_Data_lun_size.rstrip() <= 0 or f_Data_lun_count.rstrip() <= 0 or f_Log_lun_size.rstrip() <= 0 or f_Log_lun_count.rstrip() <= 0):
    print "[ERROR] Invalid LUN Size or Count......"
    sys.exit(1)
  elif ( datasize < ( memsize*1) ):
    print "[ERROR] No of Data LUNs/Sizes are not sufficient...."
    sys.exit(1)
  elif ( logsize < (memsize) ):
    print "[ERROR] No of Log LUNs/Sizes are not sufficient...."
    sys.exit(1)
  elif ( sharedsize < (memsize) ):
    print "[ERROR] No of Shared LUNs/Sizes are not sufficient...."
    sys.exit(1)

  # Getting the token and building the header with the token

  token=pvcmd_authenticate(powervc_url,auth_headers,user,password)
  auth_headers_tok={"X-Auth-Token":token, "Content-Type":"application/json", "Accept":"application/json"}

  # Retrieving the tenant id this one will be utilized in the script

  #tenant_id=pvcmd_get_tenant_id(powervc_url,auth_headers_tok,user,password)
  tenant_id="5d91abd7be654ed99ef0cfd9087ea2ab"

  # Replacing the tenant id in needed API url

  powervc_image_replaced=powervc_image.replace("<TENANT_ID>",tenant_id)
  powervc_server_replaced=powervc_server.replace("<TENANT_ID>",tenant_id)
  powervc_server_attvol_replaced=powervc_server_attvol.replace("<TENANT_ID>",tenant_id)
  powervc_placement_replaced=powervc_placement.replace("<TENANT_ID>",tenant_id)
  powervc_scg_replaced=powervc_scg.replace("<TENANT_ID>",tenant_id)
  powervc_hypervisors_replaced=powervc_hypervisors.replace("<TENANT_ID>",tenant_id)
  powervc_storage_template_replaced=powervc_storage_template.replace("<TENANT_ID>",tenant_id)
  powervc_volume_v2_replaced=powervc_volume_v2.replace("<TENANT_ID>",tenant_id)

  # Image and network ID are needed to create a VM
  # Searching the image ID

  all_images=call_api("GET",powervc_url,powervc_image_replaced,auth_headers_tok,"",200,0)
  for a_image in all_images["images"]:
    if a_image["name"] == f_image.rstrip():
      image_id=a_image["id"]
      # print "[Info] found image",f_image.rstrip(),"with id",image_id

  # Searching the network ID

  all_networks=call_api("GET", powervc_url, powervc_network, auth_headers_tok,"",200,0)
  for a_network in all_networks["networks"]:
    if a_network["name"] == f_vlan.rstrip():
      network_id=a_network["id"]
      # print "[Info] found network",f_vlan.rstrip(),"with id",network_id

  # Searching for placement

  all_aggr=call_api("GET", powervc_url,powervc_placement_replaced, auth_headers_tok,"",200,0)
  for a_aggr in all_aggr["aggregates"]:
    if a_aggr["name"] == f_host.rstrip():
      host_aggregate_id=a_aggr["id"]
      print "[Info] found host aggregation",f_host.rstrip(),"with id",host_aggregate_id

  # Searching for scg

  all_scgs=call_api("GET", powervc_url,powervc_scg_replaced, auth_headers_tok,"",200,0)
  for a_scg in all_scgs["storage_connectivity_groups"]:
    if a_scg["display_name"] == f_scg.rstrip():
      scg_id=a_scg["id"]
      print "[Info] found SCG ",f_scg.rstrip(),"with id",scg_id

  # Searching for storage template
  all_templates=call_api("GET",powervc_url,powervc_storage_template_replaced,auth_headers_tok,"",200,0)
  for a_template in all_templates["volume_types"]:
    if a_template["extra_specs"]["drivers:display_name"] == f_st.rstrip():
      st_id=a_template["id"]
      print "[Info] found storage template",f_st.rstrip(),"with id",st_id

  # JSON Body for server creation
  # New format for customer
  # Attaching a multipale volumes during initialization of boot



  # List of VOlumes

  volume_list=""

  # New Volume format for Customer

  json_body_mkvol={ "volume":
                        { "size": "<SIZE>",
                          "name": "<NAME>",
                          "volume_type": "<VOLTYPE>"
                        }
                  }

  # Attach Volume format for Customer


  json_body_attvol={ "volumeAttachment": { "volumeId": "<VOLID>", "device": "<DEVICE>"} }

  # Searching for volume in images and adding it to the json body

  all_vols=""
  powervc_image_replaced=powervc_image.replace("<TENANT_ID>",tenant_id)
  all_images=call_api("GET",powervc_url, powervc_image_replaced,auth_headers_tok,"",200,0)
  for a_image in all_images["images"]:
    if a_image["name"] == f_image.rstrip():
      image_id=a_image["id"]
      print "[Info] found image",f_image.rstrip(),"with id",image_id
      image_detail=call_api("GET",powervc_url,powervc_image_replaced+"/"+a_image["id"],auth_headers_tok,"",200,0)
      for a_vol in image_detail["image"]["metadata"]["block_device_mapping"]:
        print "[Info] found a volume with id",a_vol["volume_id"]
        # Here is the tricky part :-)
        #json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:image_volume_type_"+a_vol["volume_id"]]=st_id


  if ( f_prod_np == "prod" or f_prod_np == "Prod" ):
    print "[Info] Type ", f_prod_np
    json_body_mkvm=     {
                        "server":
                                {
                                "flavor":
                                        {
                                                "OS-FLV-EXT-DATA:ephemeral":0,
                                                "disk": 0,
                                                "extra_specs":
                                                        {
                                                                "powervm:dedicated_sharing_mode": "SHRMODE",
                                                                "powervm:dedicated_proc": "DEDPROC",
                                                                "powervm:max_vcpu": "MAXVP",
                                                                "powervm:min_mem": "MINMEM",
                                                                "powervm:max_mem": "MAXMEM",
                                                                "powervm:min_vcpu": "MINVP",
                                                                "powervm:storage_connectivity_group": "SCG"
                                                        },
                                                "ram": "<RAM>",
                                                "vcpus": "<VCPUS>",
                                                "dedicated_sharing_mode": "<DEDSHRMODE>"
                                        },
                                "imageRef": "<IMAGE_ID>",
                                "max_count": 1,
                                "availability_zone": "<HYP>",
                                "name": "<NAME>",
                                "networkRef": "<NETWORK_ID>",
                                "networks": [ { "fixed_ip": "<IP>", "uuid": "<NETWORK_ID>" } ],
                                "block_device_mapping_v2": [],
                                "user_data": "<SCRIPT>"
                                }
                         }
    # -- Setting mkvm values from buildsheet and PowerVC attributes
    json_body_mkvm["server"]["imageRef"]=image_id
    json_body_mkvm["server"]["availability_zone"]="nova:"+f_host
    json_body_mkvm["server"]["networkRef"]=network_id
    json_body_mkvm["server"]["name"]=f_name
    json_body_mkvm["server"]["networks"][0]["fixed_ip"]=f_ip
    json_body_mkvm["server"]["networks"][0]["uuid"]=network_id
    json_body_mkvol["volume"]["volume_type"]=st_id
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:dedicated_sharing_mode"]="share_idle_procs_always"
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:dedicated_proc"]="true"
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:max_vcpu"]=f_vp_max
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:min_mem"]=f_mem_min
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:max_mem"]=f_mem_max
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:min_vcpu"]=f_vp_min
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:storage_connectivity_group"]=scg_id
    json_body_mkvm["server"]["flavor"]["vcpus"]=int(f_ec)
    json_body_mkvm["server"]["flavor"]["ram"]=int(f_mem)
  else:
    print "[Info] Type ", f_prod_np
    json_body_mkvm=     {
                        "server":
                                {
                                "flavor":
                                        {
                                        "OS-FLV-EXT-DATA:ephemeral":0,
                                        "disk": 0,
                                        "extra_specs":
                                                {
                                                "powervm:proc_units": "<PROCU>",
                                                "powervm:min_proc_units": "<MINPROCU>",
                                                "powervm:max_proc_units": "<MAXPROCU>",
                                                "powervm:min_vcpu": "<MINVCPUS>",
                                                "powervm:max_vcpu": "<MAXVCPUS>",
                                                "powervm:min_mem": "<MINMEM>",
                                                "powervm:max_mem": "<MAXMEM>",
                                                "powervm:storage_connectivity_group": "SCG"
                                                },
                                        "ram": "<EAM>",
                                        "vcpus": "<VCPUS>"
                                        },
                                "imageRef": "<IMAGE_ID>",
                                "max_count": 1,
                                "availability_zone": "<HYP>",
                                "name": "<NAME>",
                                "networkRef": "<NETWORK_ID>",
                                "networks": [ { "fixed_ip": "<IP>", "uuid": "<NETWORK_ID>" } ],
                                "block_device_mapping_v2": [],
                                "user_data": "<SCRIPT>"
                                }
                        }
    # -- Setting mkvm values from buildsheet and PowerVC attributes
    json_body_mkvm["server"]["imageRef"]=image_id
    json_body_mkvm["server"]["availability_zone"]="nova:"+f_host
    json_body_mkvm["server"]["networkRef"]=network_id
    json_body_mkvm["server"]["name"]=f_name
    json_body_mkvm["server"]["networks"][0]["fixed_ip"]=f_ip
    json_body_mkvm["server"]["networks"][0]["uuid"]=network_id
    json_body_mkvol["volume"]["volume_type"]=st_id
    json_body_mkvm["server"]["flavor"]["vcpus"]=float(f_vp)
    json_body_mkvm["server"]["flavor"]["ram"]=int(f_mem)
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:proc_units"]=float(f_ec)
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:min_proc_units"]=float(f_ec_min)
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:max_proc_units"]=float(f_ec_max)
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:min_vcpu"]=int(f_vp_min)
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:max_vcpu"]=int(f_vp_max)
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:min_mem"]=int(f_mem_min)
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:max_mem"]=int(f_mem_max)
    json_body_mkvm["server"]["flavor"]["extra_specs"]["powervm:storage_connectivity_group"]=scg_id

  # Initialize arrays
  # create Additional LUNs ahead of VM deployment

  print "[Info] Creating LUNs for HANA db.."

  lun=1
  vol_uuid=[]
  vol_type=[]
  vol_size=[]
  vol_wwns=[]
  volume_list=""

  print "[Info]    Creating Data LUNs"

  while lun <= int(f_Data_lun_count):
      json_body_mkvol["volume"]["size"]=int(f_Data_lun_size)
      json_body_mkvol["volume"]["name"]=str(f_name)+"_"+str(f_Data_lun_prefix)+"_"+str(lun)
      all_volumes=call_api("POST",powervc_url,powervc_volume_v2_replaced,auth_headers_tok,json.dumps(json_body_mkvol),200,0)
      vol_id=all_volumes["volume"]["id"]
      templ = { 'uuid': vol_id, 'source_type': 'volume','destination_type': 'volume', 'boot_index': '-1' }
      time.sleep(15)
      wwn=call_api("GET",powervc_url,powervc_volume_v2_replaced + "/" + str(vol_id),auth_headers_tok,"",200,0)
      a_wwn=wwn["volume"]["metadata"]["volume_wwn"]
      vol_uuid.append(vol_id)
      vol_type.append(str(f_Data_lun_prefix))
      vol_size.append(int(f_Data_lun_size))
      vol_wwns.append(str(a_wwn))
#      json_body_mkvm["server"]["block_device_mapping_v2"].append(templ)
      volume_list=volume_list+" "+vol_id
      lun+=1

  lun=1

  print "[Info]    Creating Logs LUNs"

  while lun <= int(f_Log_lun_count):
      json_body_mkvol["volume"]["size"]=int(f_Log_lun_size)
      json_body_mkvol["volume"]["name"]=str(f_name)+"_"+str(f_Log_lun_prefix)+"_"+str(lun)
      all_volumes=call_api("POST",powervc_url,powervc_volume_v2_replaced,auth_headers_tok,json.dumps(json_body_mkvol),200,0)
      vol_id=all_volumes["volume"]["id"]
      templ = { 'uuid': vol_id, 'source_type': 'volume','destination_type': 'volume', 'boot_index': '-1' }
      time.sleep(15)
      wwn=call_api("GET",powervc_url,powervc_volume_v2_replaced + "/" + str(vol_id),auth_headers_tok,"",200,0)
      a_wwn=wwn["volume"]["metadata"]["volume_wwn"]
      vol_uuid.append(vol_id)
      vol_type.append(str(f_Log_lun_prefix))
      vol_size.append(int(f_Log_lun_size))
      vol_wwns.append(str(a_wwn))
#      json_body_mkvm["server"]["block_device_mapping_v2"].append(templ)
      volume_list=volume_list+" "+vol_id
      lun+=1
  lun=1

  print "[Info]    Creating Shared LUNs"

  while lun <= int(f_Shared_lun_count):
      json_body_mkvol["volume"]["size"]=int(f_Shared_lun_size)
      json_body_mkvol["volume"]["name"]=str(f_name)+"_"+str(f_Shared_lun_prefix)+"_"+str(lun)
      all_volumes=call_api("POST",powervc_url,powervc_volume_v2_replaced,auth_headers_tok,json.dumps(json_body_mkvol),200,0)
      vol_id=all_volumes["volume"]["id"]
      templ = { 'uuid': vol_id, 'source_type': 'volume','destination_type': 'volume', 'boot_index': '-1' }
      time.sleep(15)
      wwn=call_api("GET",powervc_url,powervc_volume_v2_replaced + "/" + str(vol_id),auth_headers_tok,"",200,0)
      a_wwn=wwn["volume"]["metadata"]["volume_wwn"]
      vol_uuid.append(vol_id)
      vol_type.append(str(f_Shared_lun_prefix))
      vol_size.append(int(f_Shared_lun_size))
      vol_wwns.append(str(a_wwn))
#      json_body_mkvm["server"]["block_device_mapping_v2"].append(templ)
      volume_list=volume_list+" "+vol_id
      lun+=1


  print "[Info] Additional volumes for HANA successfully created"
  print "------------------------------------------------------------------"
  print "[Info] Volume ID List: ", volume_list

  f=open("/tmp/"+f_name+"_vols.txt","w")
  f.write(volume_list)
  f.close()

  time.sleep(10)

  # Print Additional Storage details as per below

  print "------------------------------------------------------------------"
  #print "[Info] UUID: ", vol_uuid
  print "[Info] TYPE: ", vol_type
  print "[Info] SIZE: ", vol_size
  print "[Info] WWNs: ", vol_wwns
  # print "[Info] mkvm: ", json.dumps(json_body_mkvm)

  print "[Info] Generating activation input script"

  # Generate Post customisation script

  f=open("/tmp/"+f_name+"_disk_details.txt","w")
  i=0
  while i < len(vol_uuid):
    f.write(vol_type[i]+":"+str(vol_size[i])+":"+vol_wwns[i]+":"+f_mem+":\n")
    i+=1
  f.close()

  cmd='cp ./scripts/hanav2.sh /tmp/hanav2.'+f_name+'.sh'
  os.system( cmd )

  # replacing parameters in hanav2... script before encoding it
  # PM july 2017
  print "[Info] : f_sid value: ", f_sid
  print "[Info] : f_instance value: ", f_instance
  print "[Info] : f_email value: ", f_email
  print "[Info] : f_systempwd value: ", f_systempwd
  print "[Info] : f_Data_lun_size value: ", f_Data_lun_size
  print "[Info] : f_Log_lun_size value: ", f_Log_lun_size
  print "[Info] : f_Shared_lun_size value: ", f_Shared_lun_size
  print "[Info] : f_mem value: ", f_mem.rstrip
  inplace_change('/tmp/hanav2.'+f_name+'.sh', r'HANA_SID=', 'HANA_SID='+f_sid )
  inplace_change('/tmp/hanav2.'+f_name+'.sh', r'HANA_instance=', 'HANA_instance='+f_instance )
  inplace_change('/tmp/hanav2.'+f_name+'.sh', r'maildelivery=', 'maildelivery='+f_email )
  inplace_change('/tmp/hanav2.'+f_name+'.sh', r'SYSTEMpassword=', 'SYSTEMpassword='+f_systempwd )
  inplace_change('/tmp/hanav2.'+f_name+'.sh', r'data_disk_size=', 'data_disk_size='+f_Data_lun_size+'G' )
  inplace_change('/tmp/hanav2.'+f_name+'.sh', r'log_disk_size=', 'log_disk_size='+f_Log_lun_size+'G' )
  inplace_change('/tmp/hanav2.'+f_name+'.sh', r'shared_disk_size=', 'shared_disk_size='+f_Shared_lun_size+'G' )
  inplace_change('/tmp/hanav2.'+f_name+'.sh', r'mem_size=', 'mem_size='+f_mem )


  # disks details not needed
  #cmd='cat /tmp/'+f_name+'_disk_details.txt >> /tmp/cust_sap_'+f_name+'.sh'
  #os.system( cmd )

  base64encoded=""
  f=open("/tmp/hanav2."+f_name+".sh","r")
  base64encoded=base64.b64encode(f.read())
  f.close()

  # -- Setting script

  script=base64encoded
  json_body_mkvm["server"]["user_data"]=script
  time.sleep(10)
  print "[Info] Creating VM"

  # Create VM based on input values together with Boot LUN and additional Data and Log Luns

  create_vm=call_api("POST",powervc_url,powervc_server_replaced,auth_headers_tok,json.dumps(json_body_mkvm),200,0)
  # print "[Info] : create_vm : ",create_vm

  # Set VM ID

  vm_id=create_vm["server"]["id"]
  print "[Info] Server ID: ",vm_id
  print "[Info] Please wait - Deploying and Initializing OS Instance...."

  tenant_id="5d91abd7be654ed99ef0cfd9087ea2ab"

  powervc_server_replaced1=powervc_server.replace("<TENANT_ID>",tenant_id)
  print powervc_server_replaced1
  print powervc_server_replaced1+"/"+vm_id
  #powervc_attachment=powervc_server_replaced1+"/"+vm_id+"/"+os-volume_attachments
  server_state=call_api("GET",powervc_url,powervc_server_replaced1+"/"+vm_id,auth_headers_tok,"",200,0)
  srv_state=server_state["server"]["status"]
  # print "Server State: ",srv_state
  while srv_state != "ACTIVE":
    server_state=call_api("GET",powervc_url,powervc_server_replaced1+"/"+vm_id,auth_headers_tok,"",200,0)
    srv_state=server_state["server"]["status"]
    # print "Server State: ",srv_state
    time.sleep(20)

  print "[Info] Please wait - Attaching additional Storage...."

  tenant_id="5d91abd7be654ed99ef0cfd9087ea2ab"
  # tenant_id="f4639125c68f438fa90f312f5c2c6d4b"
  # Attach Volumes
  powervc_server_attvol_replaced1=powervc_server_attvol.replace("<TENANT_ID>",tenant_id)
  powervc_server_attvol_replaced2=powervc_server_attvol_replaced1.replace("<SERVER_ID>",vm_id)

  #x#vols = volume_list.split()
  #x#for lun_id in vols:
    #x#json_body_attvol["volumeAttachment"]["volumeId"]=lun_id
    #x#json_body_attvol["volumeAttachment"]["device"]=None
    #x#json_body_attvol["volumeAttachment"]["serverId"]=vm_id
    #x#print powervc_server_attvol_replaced1
    #x#print powervc_server_attvol_replaced2
    #x#print json_body_attvol
    #x#print json.dumps(json_body_attvol)
    #x#add_volume=call_api("POST",powervc_url,powervc_server_attvol_replaced2,auth_headers_tok,json.dumps(json_body_attvol),200,0)

  #######
  # Here is another way to attach volumes using nova
  # We must invoke ssh commands to the Powervc server to execute nova
  # cmd='source /opt/ibm/powervc/powervcrc'
  ##### Get server id
  print "[Info] server name = ",f_name
  ##### Attach volumes
  vols = volume_list.split()
  for lun_id in vols:
    print "[Info] volume id = ",lun_id
    print "[Info] server = ",f_name
    # varsetup="source /opt/ibm/powervc/powervcrc.pat"
    executeattach="source /opt/ibm/powervc/powervcrc.pat; /usr/bin/nova --os-username=sap_admin volume-attach "+f_name+" "+lun_id+" auto"
    import subprocess
    # subprocess.call(["ssh", "root@10.3.25.201", varsetup]);
    subprocess.call(["ssh", "root@10.3.25.201", executeattach]);

  print "[Info] Volumes successfully attached"
  time.sleep(15)
  print "[Info] running Activation Input Script..."
  time.sleep(60)
  print "[Info] VM Rebooting ..."
  time.sleep(10)
  print "[Info] Email sent to recipient with connection information. HANA Database Installing ......."
  time.sleep(320)
  #executetail="tail -f /var/tmp/hdblcm*"
  #import subprocess
  #subprocess.call(["sshpass -p ibm44sap", "ssh -o UserKnownHostsFile=/dev/null -oStrictHostKeyChecking=no root@10.3.25.251", executetail]);
  time.sleep(60)
  print "[Info] Instance Deployment Completed Successfully !!!!"
  time.sleep(35)
if __name__ == "__main__":
  main(sys.argv[1:])
